/*
 *  Empire - A multi-player, client/server Internet based war game.
 *  Copyright (C) 1986-2005, Dave Pare, Jeff Bailey, Thomas Ruschak,
 *                           Ken Stevens, Steve McClure
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ---
 *
 *  See the "LEGAL", "LICENSE", "CREDITS" and "README" files for all the
 *  related information and legal notices. It is expected that any future
 *  projects/authors will amend these files as needed.
 *
 *  ---
 *
 *  deli.c: Set deliveries from a sector
 * 
 *  Known contributors to this file:
 *     
 */

#include "misc.h"
#include "player.h"
#include "xy.h"
#include "sect.h"
#include "item.h"
#include "file.h"
#include "path.h"
#include "nsc.h"
#include "nat.h"
#include "commands.h"

int
deli(void)
{
    struct sctstr sect;
    int dir, del;
    struct ichrstr *ich;
    int thresh;
    int sx, sy;
    struct nstr_sect nstr;
    s_char buf[1024];
    s_char prompt[128];
    s_char *p;

    if ((ich = whatitem(player->argp[1], "deliver what? ")) == 0)
	return RET_SYN;
    /*
       if (ich->i_vtype == I_CIVIL || ich->i_vtype == I_MILIT) {
       pr("You cannot deliver people!\n");
       return RET_FAIL;
       }
     */
    if (!snxtsct(&nstr, player->argp[2]))
	return RET_SYN;

    while (nxtsct(&nstr, &sect) > 0) {
	if (!player->owner)
	    continue;

	del = sect.sct_del[ich->i_vtype];
	thresh = del & ~0x7;
	dir = del & 0x7;

	sprintf(prompt, "%s %s %s threshold or direction or 'query'? ",
		xyas(nstr.x, nstr.y, player->cnum),
		dchr[sect.sct_type].d_name, ich->i_name);
	if (!(p = getstarg(player->argp[3], prompt, buf)) || !*p)
	    return RET_SYN;
	if (*p != 'q') {
	    sprintf(prompt, "%s %s %s direction? ",
		    xyas(nstr.x, nstr.y, player->cnum),
		    dchr[sect.sct_type].d_name, ich->i_name);
	    if (((*p >= '0') && (*p <= '9')) || *p == '+') {
		thresh = atoi(p) & ~0x7;
		if (*p == '+')
		    p = NULL;
		else {
		    p = getstarg(player->argp[4], prompt, buf);
		}
	    }
	    if (p && *p) {
		dir = chkdir(*p, DIR_STOP, DIR_LAST);
		if (dir < 0)
		    return RET_SYN;
	    }

	    if (!check_sect_ok(&sect))
		continue;

	    thresh = min(thresh, ITEM_MAX) & ~7;
	    del = thresh | dir;
	    sect.sct_del[ich->i_vtype] = del;
	    putsect(&sect);
	}

	if (!del)
	    continue;

	sx = diroff[dir][0] + sect.sct_x;
	sy = diroff[dir][1] + sect.sct_y;
	pr("Deliver %s from %s @ %s to %s",
	   ich->i_name,
	   dchr[sect.sct_type].d_name,
	   xyas(sect.sct_x, sect.sct_y, player->cnum),
	   xyas(sx, sy, player->cnum));
	if (!(del & ~0x7))
	    pr("\n");
	else
	    pr(" (cutoff %d)\n", thresh);
    }

    return RET_OK;
}
